//------------------------------------------------------------------------------
//  Russian Solitaire! v.1.0
//      2006   Steph Stubler
//              University of Virginia
//              CS 216 - Programming & Data Representation [Prof. David Evans]
//
//              Card.cs
//------------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using System.Drawing;
using System.Resources;
using System.Drawing.Imaging;

namespace RussianSolitaire
{
    class Card : Label
    {
        private int value;
        private String suit;
        private bool faceUp;
        private Bitmap back;
        private int depth;
        private int whichStack;
        private bool moves;
        private bool active;
        private bool placeholder;
        private bool foundation;

        /// <summary>
        /// Creates a new playing card, needs to be added to a tableau column later.
        /// </summary>
        /// <param name="num">Value (1-13)</param>
        /// <param name="theSuit">String value of suit.</param>
        /// <param name="theBack">Bitmap of card back.</param>
        public Card(int num, String theSuit, Bitmap theBack)
        {
            Visible = false;
            setLocation(-50, -50);
            value = num;
            suit = theSuit;
            faceUp = true;
            back = theBack;
            active = false;
            placeholder = false;
            foundation = false;

            Image = getImage();

            Size cardSize = new Size(Constants.CARDWIDTH, Constants.CARDHEIGHT);
            Size = cardSize;
            AllowDrop = false;
            //Text = toString();
            BackColor = Color.Transparent;
            //BorderStyle = BorderStyle.FixedSingle;
            if ((theSuit.Equals(Constants.DIAMONDS)) || (theSuit.Equals(Constants.HEARTS)))
                ForeColor = Color.Red;
        }

        /// <summary>
        /// Creates a blank space suitable to drop Kings and/or foundation piles.
        /// </summary>
        /// <param name="x"></param>
        /// <param name="y"></param>
        public Card(int stack, int x, int y, bool found, String image)
        {
            Visible = false;
            placeholder = true;
            setLocation(x, y);
            whichStack = stack;
            moves = false;
            depth = 99;
            active = false;
            foundation = found;
            SendToBack();

            Size cardSize = new Size(Constants.CARDWIDTH, Constants.CARDHEIGHT);
            Size = cardSize;
            AllowDrop = false;
            //Text = "-";
            ResourceManager rm = new ResourceManager("RussianSolitaire.Properties.Resources", GetType().Assembly);
            Image = (Bitmap)rm.GetObject(image);
            TextAlign = ContentAlignment.MiddleCenter;
            ForeColor = Color.Black;
        }

        #region Setters
        /// <summary>
        /// Adds card to a tableau column, sets corresponding properties.
        /// </summary>
        /// <param name="up">Is card face up?</param>
        /// <param name="stack">Stack number (1-7)</param>
        /// <param name="d">Depth (0 is "top" card, and so on downward).</param>
        /// <param name="x">GUI X position of card.</param>
        /// <param name="y">GUI Y position of card.</param>
        public void addToStack(bool up, int stack, int d, int x, int y)
        {
            Visible = true;
            setLocation(x, y);
            faceUp = up;
            whichStack = stack;
            depth = d;
            moves = true;

            if (!faceUp)
            {
                AllowDrop = false;
                BackColor = Color.Gray;
                moves = false;
                Text = "";
                Image = back;
            }
        }

        /// <summary>
        /// When card moves from stack to stack or to the foundation, stack number changes.
        /// </summary>
        /// <param name="stack">New stack number.</param>
        public void changeStack(int stack)
        {
            whichStack = stack;
        }

        /// <summary>
        /// Allows or disallows other cards to be dropped onto this card.
        /// </summary>
        /// <param name="yes">True/false.</param>
        public void allowDrop(bool yes)
        {
            AllowDrop = yes;
        }

        /// <summary>
        /// Allows or disallows moving of this card.
        /// </summary>
        /// <param name="yes">True if on tableau and not faceDown, false if on foundation or is facedown.</param>
        public void allowMoving(bool yes)
        {
            moves = yes;
        }

        /// <summary>
        /// Checks to see if card's placeholder status is active.
        /// </summary>
        /// <returns>True if active</returns>
        public bool placeHolderActive()
        {
            return active;
        }

        /// <summary>
        /// Sets card's placeholder status to be active/inactive.
        /// </summary>
        /// <param name="a">True if wanted to be active.</param>
        public void setPlaceholderActive(bool a)
        {
            Visible = a;
            AllowDrop = a;
            active = a;
        }

        /// <summary>
        /// Transforms card into foundation card.
        /// </summary>
        public void setFoundation()
        {
            placeholder = true;
            active = true;
            foundation = true;
            moves = false;
            AllowDrop = true;
            BringToFront();
        }

        public void changeDepth(int x)
        {
            depth = x;
            // Text = depth.ToString() + " (" + whichStack + ")";
        }

        public void changeDeck(Bitmap newBack)
        {
            back = newBack;
        }

        public void setLocation(int x, int y)
        {
            Location = new Point(x, y);
        }

        public void setBackColor(Color c)
        {
            BackColor = c;
        }
        #endregion

        #region Getters
        /// <summary>
        /// Checks if card accepts other cards to be dropped onto it.
        /// </summary>
        /// <returns>True/false.</returns>
        public bool allowsDrop()
        {
            return AllowDrop;
        }

        /// <summary>
        /// Checks to see if card is allowed to move.
        /// </summary>
        /// <returns>If card can move.</returns>
        public bool canMove()
        {
            return moves;
        }

        public String getSuit()
        {
            return suit;
        }

        public int getValue()
        {
            return value;
        }

        public int getStack()
        {
            return whichStack;
        }

        public int getDepth()
        {
            return depth;
        }

        /// <summary>
        /// Checks to see if card is on the foundation.
        /// </summary>
        /// <returns>True if on foundation.</returns>
        public bool isFoundation()
        {
            return foundation;
        }

        /// <summary>
        /// Checks to see if card is merely a placeholder.
        /// </summary>
        /// <returns>True if placeholder</returns>
        public bool isPlaceholder()
        {
            return placeholder;
        }

        public bool isFaceUp()
        {
            return faceUp;
        }

        /// <summary>
        /// Finds corresponding .png image associated with a card's value and suit.
        /// </summary>
        /// <returns>Bitmap version of corresponding .png image.</returns>
        public Bitmap getImage()
        {
            ResourceManager rm = new ResourceManager("RussianSolitaire.Properties.Resources", GetType().Assembly);
            Bitmap b;
            String s;
            if (!faceUp)
            {
                b = back;
            }
            else
            {
                if (suit.Equals(Constants.HEARTS))
                {
                    s = "h" + value;
                }
                else if (suit.Equals(Constants.DIAMONDS))
                {
                    s = "d" + value;
                }

                else if (suit.Equals(Constants.CLUBS))
                {
                    s = "c" + value;
                }

                else
                {
                    s = "s" + value;
                }

                b = (Bitmap)rm.GetObject(s);
            }
            return b;
        }
        #endregion

        public void flip()
        {
            faceUp = !faceUp;
            moves = faceUp;
            AllowDrop = faceUp;
            BackColor = Color.White;
            //Text = toString();
            Image = getImage();
        }

        /// <summary>
        /// For debugging purposes, converts card's value and suit to a String.
        /// </summary>
        /// <returns>String of card.</returns>
        public String toString()
        {
            String s;
            if (placeholder)
            {
                s = "-";
            }
            else
            {
                if (value == 1)
                    s = "A";
                else if (value == 11)
                    s = "J";
                else if (value == 12)
                    s = "Q";
                else if (value == 13)
                    s = "K";
                else
                    s = value.ToString();
                s += " (" + suit.Substring(0, 1).ToUpper() + ")";
            }
            return s;
        }
    }
}
